(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('prop-types')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react', 'prop-types'], factory) :
	(factory((global['react-portalgun'] = {}),global.React,global.PropTypes));
}(this, (function (exports,React,PropTypes) { 'use strict';

React = React && React.hasOwnProperty('default') ? React['default'] : React;
PropTypes = PropTypes && PropTypes.hasOwnProperty('default') ? PropTypes['default'] : PropTypes;

/**
 * Copyright 2014-2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

/**
 * Similar to invariant but only logs a warning if the condition is not met.
 * This can be used to log issues in development environments in critical
 * paths. Removing the logging code for production environments will keep the
 * same logic and follow the same code paths.
 */

var __DEV__ = process.env.NODE_ENV !== 'production';

var warning = function() {};

if (__DEV__) {
  warning = function(condition, format, args) {
    var len = arguments.length;
    args = new Array(len > 2 ? len - 2 : 0);
    for (var key = 2; key < len; key++) {
      args[key - 2] = arguments[key];
    }
    if (format === undefined) {
      throw new Error(
        '`warning(condition, format, ...args)` requires a warning ' +
        'message argument'
      );
    }

    if (format.length < 10 || (/^[s\W]*$/).test(format)) {
      throw new Error(
        'The warning format should be able to uniquely identify this ' +
        'warning. Please, use a more descriptive format than: ' + format
      );
    }

    if (!condition) {
      var argIndex = 0;
      var message = 'Warning: ' +
        format.replace(/%s/g, function() {
          return args[argIndex++];
        });
      if (typeof console !== 'undefined') {
        console.error(message);
      }
      try {
        // This error was thrown as a convenience so that you can use this stack
        // to find the callsite that caused this warning to fire.
        throw new Error(message);
      } catch(x) {}
    }
  };
}

var warning_1 = warning;

const withPortal = (
  portalType,
  callbackName,
  mapPropsToPortal,
) => WrappedComponent => {
  const Wrapper = (props, context) => {
    let callback;

    if (!context.showPortal) {
      warning_1(
        false,
        'Error: portalgun. Wrap your application in a <PortalProvider />',
      );
      return React.createElement(WrappedComponent, props);
    }

    if (mapPropsToPortal) {
      callback = event => {
        const portalProps = mapPropsToPortal({ event, ownProps: props });
        return context.showPortal(portalType, portalProps);
      };
    } else {
      callback = event => {
        if (event.persist) {
          event.persist();
        }

        context.showPortal(portalType, event);
      };
    }

    return React.createElement(
      WrappedComponent,
      Object.assign({}, props, { [callbackName || 'showPortal']: callback }),
    );
  };

  Wrapper.contextTypes = {
    showPortal: PropTypes.func,
  };

  return Wrapper;
};

class PortalProvider extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      portalType: null,
      portalProps: {},
    };
  }

  getChildContext() {
    return { showPortal: this.showPortal.bind(this) };
  }

  showPortal(type, props) {
    this.setState({ portalType: type, portalProps: props });
  }

  closePortal() {
    this.setState({ portalType: null, portalProps: {} });
  }

  render() {
    const { portals } = this.props;
    const { portalType, portalProps } = this.state;
    const PortalComponent = portals[portalType];

    warning_1(
      !(portalType && !PortalComponent),
      `Warning: portalgun. ${portalType} is not a valid element`,
    );

    return React.createElement(
      'div',
      null,
      PortalComponent &&
        React.createElement(
          PortalComponent,
          Object.assign({}, portalProps, {
            onClose: this.closePortal.bind(this),
          }),
        ),
      this.props.children,
    );
  }
}

PortalProvider.childContextTypes = {
  showPortal: PropTypes.func,
};

exports.withPortal = withPortal;
exports.PortalProvider = PortalProvider;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=react-portalgun.umd.js.map
